package edsminer.clock;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.Timestamp;
import java.util.Date;
import edsminer.general.Tran;

/**
 * Thread-safe version of the ClockEngine class
 * 
 * @author Eric Yuan
 *
 */
public class ClockEngineTS implements Clock {
//	private static Connection connect = null;
//	private static PreparedStatement initStmt = null;
//	private static PreparedStatement wrapStmt = null;
//	private Date initDate;
//	private Date termDate;
//	private Timestamp tmstpInit;
//	private Timestamp tmstpTerm;
//	private String tranID;
//	private String seqID;
	
	public ClockEngineTS(){
		super();
	}

	@Override
	/***********************************************************************************
	 * executeTransaction()
	 * 
	 * This method is the functional public method that other components invoke to use this component.
	 * The method will receive a transaction object identifying the id of the transaction.
	 * The method will attempt to log entry into the method in the TRAN_INIT table.
	 * The method will attempt to log exit from the method in the TRAN_TERM table.
	 * Between these logging function, the method may perform some functions, possibly including
	 * using other components.
	 **********************************************************************************/
	public <T> T executeTransaction(Tran<T> t) throws RemoteException {
		
		Connection connect = null;
		PreparedStatement initStmt = null;
		PreparedStatement wrapStmt = null;
		Date initDate = null;
		Date termDate = null;
		Timestamp tmstpInit = null;
		Timestamp tmstpTerm = null;
		String tranID = null;
		String seqID = null;

		try {
			/* ----- Try to log the entry into the method ----- */
			// Setup the connection with the DB
			connect = DriverManager.getConnection("jdbc:mysql://127.0.0.1/test?" + "user=edsmineman&password=edspass");
			// create the prepared statement for the transaction_init table insertion
			initStmt = connect.prepareStatement("insert into TEST.TRAN_INIT (id, tran_init_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?)");
			// create the prepared statement for the transaction_wrap table insertion
			wrapStmt = connect.prepareStatement("insert into TEST.TRAN_WRAP (id, tran_init_tmstp_l, tran_term_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?, ?)");

			// set the entry timestamp
			initDate = new Date();
			tmstpInit = new java.sql.Timestamp(initDate.getTime());
			// set the id value for the transaction
			tranID = (String) t.getID();
			seqID = (String) t.getSeq();
			
			// set the value for tran_init_tmstp and tran_id
			initStmt.setLong(1, tmstpInit.getTime());
			initStmt.setString(2, tranID);
			initStmt.setString(3, seqID);
			// execute the SQL insert statement
			initStmt.executeUpdate();
		
			/* ----- Perform the work of the method here ----- */
			System.out.println("ClockEngine executing its business logic.");
			try {Thread.sleep(250);
			} catch (Exception e) {
				System.err.println("ClockEngine Sleep exception:");
				e.printStackTrace();
			}

			System.out.println("Returning without calling other components.");
		
			/* ----- Try to log the exit from the method ----- */
			
			// set the exit timestamp
			termDate = new Date();
			tmstpTerm = new java.sql.Timestamp(termDate.getTime());
			
			// set the value for tran_init_tmstp, tran_term_tmstp, tran_id respectively
			wrapStmt.setLong(1, tmstpInit.getTime());
			wrapStmt.setLong(2, tmstpTerm.getTime());
			wrapStmt.setString(3, tranID);
			wrapStmt.setString(4, seqID);
			// execute the SQL insert statement
			wrapStmt.executeUpdate();
			
			connect.close();
		} catch (Exception e) {
			System.err.println("ClockEngine server exception:");
			e.printStackTrace();
		}
		
		// exit from the method
		return (T) seqID;
	}

	
	/***********************************************************************************
	 * main()
	 * 
	 * This method runs when the object is first started up.
	 * This method should do all of the basic one-time setup for the object.
	 * This object will remain active because it will be referenced by the RMI registry.
	 **********************************************************************************/
	public static void main(String[] args) {
		if (System.getSecurityManager() == null){
			System.setSecurityManager(new SecurityManager());
		}
		
		/* Try to register this object in the RMI registry */
		try {
			String name = "rmi://EDSMine/Clock";
			Clock engine = new ClockEngineTS();
			Clock stub = (Clock) UnicastRemoteObject.exportObject(engine, 0);
			Registry registry = LocateRegistry.getRegistry();
			registry.rebind(name, stub);
			System.out.println("ClockEngine bound");
		} catch (Exception e) {
			System.err.println("ClockEngine RMI registration exception:");
			e.printStackTrace();
		}
		
		/* Try to connect to the database */
		try {
			// This will load the MySQL driver, each DB has its own driver
			Class.forName("com.mysql.jdbc.Driver");
//			// Setup the connection with the DB
//			connect = DriverManager.getConnection("jdbc:mysql://127.0.0.1/test?" + "user=edsmineman&password=edspass");
//			// create the prepared statement for the transaction_init table insertion
//			initStmt = connect.prepareStatement("insert into TEST.TRAN_INIT (id, tran_init_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?)");
//			// create the prepared statement for the transaction_wrap table insertion
//			wrapStmt = connect.prepareStatement("insert into TEST.TRAN_WRAP (id, tran_init_tmstp_l, tran_term_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?, ?)");
			
		} catch (Exception e) {
			System.err.println("ClockEngine DB exception:");
			e.printStackTrace();
		}
	}
}
