package edsminer.hqui;

import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import edsminer.general.codeController;
import edsminer.map.Map;
import edsminer.deploymentAdvisor.DeploymentAdvisor;
import edsminer.general.edsTran;
import edsminer.strategyAnalysis.StrategyAnalysis;
import edsminer.simulationAgent.SimulationAgent;


//import edsminer.repository.Repository;
import java.util.Random;

/**
 * Modified to execute the following use cases:
 * 		StrategyAnalysis
 * 		SimulationAgent
 * 		DeploymentAdvisor
 * 		MapView
 * If the 3rd argument is set, the main method also tries to inject abnormal transactions
 * based on predefined anomaly rate.
 * 
 * @author Eric Yuan
 *
 */
public class Hqui {
	
//	private static final String mapName = "rmi://EDSMine/Map";
//	private static final String repositoryName = "rmi://EDSMine/Repository";
	private static final String mapName = "rmi://EDSMine/MapView";
	private static final String deploymentadvisorName = "rmi://EDSMine/DeploymentAdvisor";
	private static final String simulationagentName = "rmi://EDSMine/SimulationAgent";
	private static final String simulationagentanomalyName = "rmi://EDSMine/SimulationAgentAnomaly";
	private static final String strategyanalysisName = "rmi://EDSMine/StrategyAnalysis";
	private static final String strategyanalysisanomalyName = "rmi://EDSMine/StrategyAnalysisAnomaly";
	
	private static final double ANOMALY_INTERVAL = 3.0;		// 3-sigma = 0.3% anomalies
	
	private static boolean insertAnomaly = false;
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (System.getSecurityManager() == null){
			System.setSecurityManager(new SecurityManager());
		}
		
		if (args.length < 2) {
			System.out.println("Usage: Hqui <registryHost> <userID> [ 1 ]");
			System.exit(1);
		}
		
		if (args.length > 2 && args[2].equals("1")) {
			System.out.println("HQUI will add anomaly transactions.");
			insertAnomaly = true;
		}

		
		try{
			/* Get some controller parameters */
			codeController cont = new codeController();
			String sessionID = args[1];
			sessionID = sessionID.trim();
			
			/* Get a hold of the RMI components */
			Registry registry = LocateRegistry.getRegistry(args[0]);
			Map map = (Map) registry.lookup(mapName);
			DeploymentAdvisor deploymentAdvisor = null;
			SimulationAgent simulationAgent = null;
			SimulationAgent simulationAgentAnomaly = null;
			StrategyAnalysis strategyAnalysis = null;
			StrategyAnalysis strategyAnalysisAnomaly = null;
			
			if (cont.sooper()){
				deploymentAdvisor = (DeploymentAdvisor) registry.lookup(deploymentadvisorName);
				simulationAgent = (SimulationAgent) registry.lookup(simulationagentName);
				simulationAgentAnomaly = (SimulationAgent) registry.lookup(simulationagentanomalyName);
				strategyAnalysis = (StrategyAnalysis) registry.lookup(strategyanalysisName);
				strategyAnalysisAnomaly = (StrategyAnalysis) registry.lookup(strategyanalysisanomalyName);
			}

			/* Create a transaction object */
			edsTran tran = new edsTran();
			
			/* Start the transaction loop */
			Random ran1 = new Random();
			Random ran2 = new Random();
			Random ran3 = new Random();
			int ranNum = 0, loops = 5000, cap = 2, sleepCap = 5, ranFixed = 0;
			double ranSleep = 0, ranSleep1 = 0, ranSleep2 = 0;
			if (cont.sooper()) {cap = 4;}
			for(int i = 0; i < loops; i++){
				ranNum = ran1.nextInt(cap);
				tran.setSeq(sessionID + "_" + i);
				
				// choose the sleep time
				ranSleep1 = ran2.nextGaussian() + 1.0;
				ranSleep2 = ran3.nextGaussian() + 1.0;
				ranSleep = Math.sqrt((ranSleep1 * ranSleep1) + (ranSleep2 * ranSleep2));
				//ranSleep = 0;	
				
				System.out.println("User thinking for " + ranSleep + " seconds.");
				try {Thread.sleep(((int) Math.round(ranSleep)) * 1000);
				} catch (Exception e) {
					System.err.println("MapEngine Sleep exception:");
					e.printStackTrace();
				}
				
				// Insert anomalous transaction, if desired
				double ranGauss = ran2.nextGaussian();
				if (insertAnomaly && (ranGauss > ANOMALY_INTERVAL || ranGauss < - ANOMALY_INTERVAL)) {
//					System.out.println(i + " of " + loops + ": !!! Calling the StrategyAnalysisAnomaly component.");
//					tran.setID("HQUItoSTRATEGYANALYSIS");
//					strategyAnalysisAnomaly.executeTransaction(tran);
					if (ranNum == 0 || ranNum == 1) {
						System.out.println(i + " of " + loops + ": !!! Calling the StrategyAnalysisAnomaly component.");
						tran.setID("HQUItoSTRATEGYANALYSIS");
						strategyAnalysisAnomaly.executeTransaction(tran);
					}
					else {
						System.out.println(i + " of " + loops + ": !!! Calling the SimulationAgentAnomaly component.");
						tran.setID("HQUItoSIMULATIONAGENT");
						simulationAgentAnomaly.executeTransaction(tran);
					}
				}else if (ranNum == 0){
					System.out.println(i + " of " + loops + ": Calling the Map component.");
					tran.setID("HQUItoMAP");
					map.executeTransaction(tran);
				}else if (ranNum == 1){
					System.out.println(i + " of " + loops + ": Calling the DeploymentAdvisor component.");
					tran.setID("HQUItoDEPLOYMENTADVISOR");
					deploymentAdvisor.executeTransaction(tran);
				}else if (ranNum == 2){
					System.out.println(i + " of " + loops + ": Calling the SimulationAgent component.");
					tran.setID("HQUItoSIMULATIONAGENT");
					simulationAgent.executeTransaction(tran);
				}else if (ranNum == 3){
					System.out.println(i + " of " + loops + ": Calling the StrategyAnalysis component.");
					tran.setID("HQUItoSTRATEGYANALYSIS");
					strategyAnalysis.executeTransaction(tran);
				}
			}
		} catch (Exception e){
			System.err.println("Hqui exception:");
			e.printStackTrace();
		}
	}
}
