package edsminer.hqui2;

import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import edsminer.map.Map;
import edsminer.repository.Repository;
import edsminer.general.codeController;
import edsminer.general.edsTran;
import edsminer.strategyAnalysis.StrategyAnalysis;
import edsminer.simulationAgent.SimulationAgent;

import java.util.Random;

public class Hqui2 {
	
	private static final String mapName = "rmi://EDSMine/Map";
	private static final String repositoryName = "rmi://EDSMine/Repository";
	private static final String simulationagentName = "rmi://EDSMine/SimulationAgent";
	private static final String strategyanalysisName = "rmi://EDSMine/StrategyAnalysis";
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (System.getSecurityManager() == null){
			System.setSecurityManager(new SecurityManager());
		}
		
		try{
			/* Get some controller parameters */
			codeController cont = new codeController();
			
			/* Get a hold of the RMI components */
			Registry registry = LocateRegistry.getRegistry(args[0]);
			Map map = (Map) registry.lookup(mapName);
			Repository repository = (Repository) registry.lookup(repositoryName);
			SimulationAgent simulationAgent = null;
			StrategyAnalysis strategyAnalysis = null;
			if (cont.sooper()){
				simulationAgent = (SimulationAgent) registry.lookup(simulationagentName);
				strategyAnalysis = (StrategyAnalysis) registry.lookup(strategyanalysisName);
			}

			/* Create a transaction object */
			edsTran tran = new edsTran();
			
			/* Start the transaction loop */
			Random ran1 = new Random();
			Random ran2 = new Random();
			Random ran3 = new Random();
			int ranNum = 0, loops = 5000, cap = 2, sleepCap = 5;
			double ranSleep = 0, ranSleep1 = 0, ranSleep2 = 0;
			if (cont.sooper()) {cap = 4;}
			for(int i = 0; i < loops; i++){
				ranNum = ran1.nextInt(cap);
				tran.setSeq("2_" + i);
				
				// choose the sleep time
				ranSleep1 = ran2.nextGaussian() + 1.5;
				ranSleep2 = ran3.nextGaussian() + 1.5;
				ranSleep = Math.sqrt((ranSleep1 * ranSleep1) + (ranSleep2 * ranSleep2));
					
				System.out.println("User thinking for " + ranSleep + " seconds.");
				try {Thread.sleep(((int) Math.round(ranSleep)) * 1000);
				} catch (Exception e) {
					System.err.println("MapEngine Sleep exception:");
					e.printStackTrace();
				}
				
				if (ranNum == 0){
					System.out.println(i + " of " + loops + ": Calling the Map component.");
					tran.setID("HQUItoMAP");
					map.executeTransaction(tran);
				}else if (ranNum == 1){
					System.out.println(i + " of " + loops + ": Calling the Repository component.");
					tran.setID("HQUItoREPOSITORY");
					repository.executeTransaction(tran);
				}else if (ranNum == 2){
					System.out.println(i + " of " + loops + ": Calling the SimulationAgent component.");
					tran.setID("HQUItoSIMULATIONAGENT");
					simulationAgent.executeTransaction(tran);
				}else if (ranNum == 3){
					System.out.println(i + " of " + loops + ": Calling the StrategyAnalysis component.");
					tran.setID("HQUItoSTRATEGYANALYSIS");
					strategyAnalysis.executeTransaction(tran);
				}
			}
		} catch (Exception e){
			System.err.println("Hqui2 exception:");
			e.printStackTrace();
		}
	}
}
