package edsminer.resourceManager;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.Timestamp;
import java.util.Date;
import java.util.Random;

import edsminer.general.Tran;
import edsminer.general.codeController;
import edsminer.map.Map;
import edsminer.repository.Repository;
import edsminer.resourceManager.ResourceManager;
import edsminer.resourceMonitor.ResourceMonitor;

/**
 * Refactored to call Map and Repository components
 * 
 * Thread-safe version of the ResourceManagerEngine class.
 * 
 * @author Eric Yuan
 *
 */
public class ResourceManagerEngineTS implements ResourceManager {
	private static codeController cont = new codeController();
	private static boolean setupSuccessful = true;
	private static Registry registry = null;
//	private static Connection connect = null;
//	private static PreparedStatement initStmt = null;
//	private static PreparedStatement wrapStmt = null;
//	private Date initDate;
//	private Date termDate;
//	private Timestamp tmstpInit;
//	private Timestamp tmstpTerm;
//	private String tranID;
//	private String seqID;
	// start sooper
	private static final String repositoryName = "rmi://EDSMine/Repository";
	private static final String resourceMonitorName = "rmi://EDSMine/ResourceMonitor";
	private static final String mapName = "rmi://EDSMine/Map";
	private static Repository repository = null;
	private static ResourceMonitor resourceMonitor = null;
	private static Map map = null;
	// end sooper
	private Random ran = new Random(42);
	private int ranNum = 0;
	private static int cap;
	
	public ResourceManagerEngineTS(){
		super();
	}

	@Override
	/***********************************************************************************
	 * executeTransaction()
	 * 
	 * This method is the functional public method that other components invoke to use this component.
	 * The method will receive a transaction object identifying the id of the transaction.
	 * The method will attempt to log entry into the method in the TRAN_INIT table.
	 * The method will attempt to log exit from the method in the TRAN_TERM table.
	 * Between these logging function, the method may perform some functions, possibly including
	 * using other components.
	 **********************************************************************************/
	public <T> T executeTransaction(Tran<T> t) throws RemoteException {
		
		Connection connect = null;
		PreparedStatement initStmt = null;
		PreparedStatement wrapStmt = null;
		Date initDate = null;
		Date termDate = null;
		Timestamp tmstpInit = null;
		Timestamp tmstpTerm = null;
		String tranID = null;
		String seqID = null;
		String userProfile = null;

		/* Try to get a hold of components needed through RMI if it didn't work before */
		if (!setupSuccessful){
			try {
				// start sooper
				if (cont.sooper()){
					if (repository == null) {repository = (Repository) registry.lookup(repositoryName);}
					if (resourceMonitor == null) {resourceMonitor = (ResourceMonitor) registry.lookup(resourceMonitorName);}
					if (map == null) {map = (Map) registry.lookup(mapName);}
				}// end sooper
				setupSuccessful = true;
			} catch (Exception e) {
				System.err.println("ResourceManager RMI lookup exception:");
				e.printStackTrace();
			}
		}
		
		try {
			/* ----- Try to log the entry into the method ----- */
			// Setup the connection with the DB
			connect = DriverManager.getConnection("jdbc:mysql://127.0.0.1/test?" + "user=edsmineman&password=edspass");
			// create the prepared statement for the transaction_init table insertion
			initStmt = connect.prepareStatement("insert into TEST.TRAN_INIT (id, tran_init_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?)");
			// create the prepared statement for the transaction_wrap table insertion
			wrapStmt = connect.prepareStatement("insert into TEST.TRAN_WRAP (id, tran_init_tmstp_l, tran_term_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?, ?)");

			// set the entry timestamp
			initDate = new Date();
			tmstpInit = new java.sql.Timestamp(initDate.getTime());
			// set the id value for the transaction
			tranID = (String) t.getID();
			seqID = (String) t.getSeq();
			userProfile = (String) t.getUserProfile();
			
			// set the value for tran_init_tmstp and tran_id
			initStmt.setLong(1, tmstpInit.getTime());
			initStmt.setString(2, tranID);
			initStmt.setString(3, seqID);
			// execute the SQL insert statement
			initStmt.executeUpdate();
				
			/* ----- Perform the work of the method here ----- */

			// 3/4 chance to call the ResourceMonitor component
			// TODO - better way to handle this instead of hardcoding. Basically makes sure the anomaly event  
			//        STRATEGYANALYSIStoRESOURCEMANAGER does not call ResourceMonitor, according to design
			ranNum = ran.nextInt(cap*2);
			if (tranID.equals("DEPLOYMENTADVISORtoRESOURCEMANAGER") && ranNum != 0) {
				System.out.println("Calling the ResourceMonitor component.");
				t.setID((T) "RESOURCEMANAGERtoRESOURCEMONITOR");
				resourceMonitor.executeTransaction(t);
			}
		
			// 1/2 chance to call the Map component
			// TODO - better way to handle this instead of hardcoding. Basically makes sure the anomaly event  
			//        STRATEGYANALYSIStoRESOURCEMANAGER does not call Map, according to design
			ranNum = ran.nextInt(cap);
			if (tranID.equals("DEPLOYMENTADVISORtoRESOURCEMANAGER") && ranNum != 0) {
				System.out.println("Calling the Map component.");
				t.setID((T) "RESOURCEMANAGERtoMAP");
				map.executeTransaction(t);
			}
			
			System.out.println("ResourceManagerEngine performing its business logic.");
			int thinkTime = userProfile.equals(Tran.USER_EXPERT) ? 250 : 1250;
			try {Thread.sleep(thinkTime + 250 * ran.nextInt(2));
			} catch (Exception e) {
				System.err.println("ResourceManagerEngine Sleep exception:");
				e.printStackTrace();
			}
			
			// Call the repository component
			
			// 3/4 chance to call the Repository component
			ranNum = ran.nextInt(cap*2);
			if (ranNum == 1 || ranNum == 2 || ranNum == 3) {
				System.out.println("Calling the Repository component.");
				t.setID((T) "RESOURCEMANAGERtoREPOSITORY");
				repository.executeTransaction(t);
			}
		
			/* ----- Try to log the exit from the method ----- */
			// set the exit time stamp
			termDate = new Date();
			tmstpTerm = new java.sql.Timestamp(termDate.getTime());
			
			// set the value for tran_init_tmstp, tran_term_tmstp, tran_id respectively
			wrapStmt.setLong(1, tmstpInit.getTime());
			wrapStmt.setLong(2, tmstpTerm.getTime());
			wrapStmt.setString(3, tranID);
			wrapStmt.setString(4, seqID);
			// execute the SQL insert statement
			wrapStmt.executeUpdate();
			
			connect.close();
		} catch (Exception e) {
			System.err.println("ResourceManagerEngine server exception:");
			e.printStackTrace();
		}
		
		// exit from the method
		return (T) seqID;
	}

	
	/***********************************************************************************
	 * main()
	 * 
	 * This method runs when the object is first started up.
	 * This method should do all of the basic one-time setup for the object.
	 * This object will remain active because it will be referenced by the RMI registry.
	 **********************************************************************************/
	public static void main(String[] args) {
		if (System.getSecurityManager() == null){
			System.setSecurityManager(new SecurityManager());
		}
		
		cap = 1;
		// start sooper
		if (cont.sooper()) {cap = 2;}
		// end sooper
		
		/* Try to register this object in the RMI registry */
		try {
			String name = "rmi://EDSMine/ResourceManager";
			ResourceManager engine = new ResourceManagerEngineTS();
			ResourceManager stub = (ResourceManager) UnicastRemoteObject.exportObject(engine, 0);
			Registry registry = LocateRegistry.getRegistry();
			registry.rebind(name, stub);
			System.out.println("ResourceManagerEngine bound");
		} catch (Exception e) {
			System.err.println("ResourceManagerEngine RMI exception:");
			e.printStackTrace();
		}
		
		/* Try to connect to the database */
		try {
			// This will load the MySQL driver, each DB has its own driver
			Class.forName("com.mysql.jdbc.Driver");
//			// Setup the connection with the DB
//			connect = DriverManager.getConnection("jdbc:mysql://127.0.0.1/test?" + "user=edsmineman&password=edspass");
//			// create the prepared statement for the transaction_init table insertion
//			initStmt = connect.prepareStatement("insert into TEST.TRAN_INIT (id, tran_init_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?)");
//			// create the prepared statement for the transaction_wrap table insertion
//			wrapStmt = connect.prepareStatement("insert into TEST.TRAN_WRAP (id, tran_init_tmstp_l, tran_term_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?, ?)");
			
		} catch (Exception e) {
			System.err.println("ResourceManagerEngine DB exception:");
			e.printStackTrace();
		}
		
		/* Try to get a hold of components needed through RMI */
		try {
			registry = LocateRegistry.getRegistry(args[0]);
			// start sooper
			if (cont.sooper()){
				repository = (Repository) registry.lookup(repositoryName);
				resourceMonitor = (ResourceMonitor) registry.lookup(resourceMonitorName);
				map = (Map) registry.lookup(mapName);
			}// end sooper
		} catch (Exception e) {
			// start update
			setupSuccessful = false;
			// end update
			System.err.println("ResourceManager RMI lookup exception:");
			e.printStackTrace();
		}
	}
}
