package edsminer.strategyAnalysis;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.Timestamp;
import java.util.Date;
import java.util.Random;
import java.util.Vector;

import edsminer.general.Tran;
import edsminer.general.codeController;
import edsminer.map.Map;
import edsminer.clock.Clock;
import edsminer.resourceManager.ResourceManager;
import edsminer.strategyKB.StrategyKB;
import edsminer.weatherAnalyzer.WeatherAnalyzer;

/**
 * Implements an anomalous transaction (Hijacked StrategyAnalysis component)
 * 
 * @author Eric Yuan
 *
 */
public class StrategyAnalysisAnomalyEngine implements StrategyAnalysis {
	private static codeController cont = new codeController();
	private static boolean setupSuccessful = true;
	private static Registry registry = null;
	private static Connection connect = null;
	private static PreparedStatement initStmt = null;
	private static PreparedStatement wrapStmt = null;
	// start sooper
	private static final String clockName = "rmi://EDSMine/Clock";
	private static final String mapName = "rmi://EDSMine/Map";
	private static final String weatherAnalyzerName = "rmi://EDSMine/WeatherAnalyzer";
	private static final String strategyKBName = "rmi://EDSMine/StrategyKB";
	private static final String resourceManagerName = "rmi://EDSMine/ResourceManager";
	// end sooper
	private Random ran = new Random(42);
	// start sooper
	private static Clock clock = null;
	private static Map map = null;
	private static WeatherAnalyzer weatherAnalyzer = null;
	private static StrategyKB strategyKB = null;
	private static ResourceManager resourceManager = null;
	
	// end sooper
	private int ranNum = 0;
	private Date initDate;
	private Date termDate;
	private Timestamp tmstpInit;
	private Timestamp tmstpTerm;
	private String tranID;
	private String seqID;
	private static int cap;
	private static int init_pos;
	private Vector<String> tran_vec = new Vector();
	private Vector<String> seq_vec = new Vector();
	private Vector<java.sql.Timestamp> tmstp_vec = new Vector();
	
	public StrategyAnalysisAnomalyEngine(){
		super();
	}

	@Override
	/***********************************************************************************
	 * executeTransaction()
	 * 
	 * This method is the functional public method that other components invoke to use this component.
	 * The method will receive a transaction object identifying the id of the transaction.
	 * The method will attempt to log entry into the method in the TRAN_INIT table.
	 * The method will attempt to log exit from the method in the TRAN_TERM table.
	 * Between these logging function, the method may perform some functions, possibly including
	 * using other components.
	 **********************************************************************************/
	public <T> T executeTransaction(Tran<T> t) throws RemoteException {
		
		/* Try to get a hold of components needed through RMI if it didn't work before */
		if (!setupSuccessful){
			try {
				// start sooper
				if (cont.sooper()){
					if (clock == null) {clock = (Clock) registry.lookup(clockName);}
					if (map == null) {map = (Map) registry.lookup(mapName);}
					if (resourceManager == null) {resourceManager = (ResourceManager) registry.lookup(resourceManagerName);}
//					if (weatherAnalyzer == null) {weatherAnalyzer = (WeatherAnalyzer) registry.lookup(weatherAnalyzerName);}
//					if (strategyKB == null) {strategyKB = (StrategyKB) registry.lookup(strategyKBName);}
				}// end sooper
				setupSuccessful = true;
			} catch (Exception e) {
				System.err.println("StrategyAnalysisAnomalyEngine RMI lookup exception:");
				e.printStackTrace();
			}
		}
		
		/* Try to log the entry into the method */
		try {
			// set the entry timestamp
			initDate = new Date();
			tmstpInit = new java.sql.Timestamp(initDate.getTime());
			// set the id value for the transaction
			tranID = (String) t.getID();
			seqID = (String) t.getSeq();
			
			// set the value for tran_init_tmstp and tran_id
			initStmt.setLong(1, tmstpInit.getTime());
			initStmt.setString(2, tranID);
			initStmt.setString(3, seqID);
			// execute the SQL insert statement
			initStmt.executeUpdate();
			
			// store off the values for return later on
			//tran_vec.add(tranID);
			//seq_vec.add(seqID);
			//tmstp_vec.add(tmstpInit);
		} catch (Exception e) {
			System.err.println("StrategyAnalysisAnomalyEngine LogEntry exception:");
			e.printStackTrace();
		}
		
		// 3/4 chance we call clock
		ranNum = ran.nextInt(4);
		if (ranNum != 0){
			System.out.println("Calling the Clock component.");
			t.setID((T) "STRATEGYANALYSIStoCLOCK");
			seqID = (String) clock.executeTransaction(t);
		}
		
		// 3/4 chance we call map
		ranNum = ran.nextInt(4);
		if (ranNum != 0){
			System.out.println("Calling the Map component.");
			t.setID((T) "STRATEGYANALYSIStoMAP");
			seqID = (String) map.executeTransaction(t);
		}
		
		// ANOMALOUS BEHAVIOR: Call ResourceManager
		ranNum = ran.nextInt(4);
		if (ranNum != 0){
			System.out.println("Calling the ResourceManager component.");
			t.setID((T) "STRATEGYANALYSIStoRESOURCEMANAGER");
			seqID = (String) resourceManager.executeTransaction(t);
		}
		
		// 1/4 chance to call each of 3 components or call nothing
//		ranNum = ran.nextInt(cap);
//		if (ranNum == 0){
//			System.out.println("Calling the Clock component.");
//			t.setID((T) "STRATEGYANALYSIStoCLOCK");
//			seqID = (String) clock.executeTransaction(t);
//		}else if (ranNum == 1){
//			System.out.println("Calling the SimulationAgent component.");
//			t.setID((T) "STRATEGYANALYSIStoSIMULATIONAGENT");
//			seqID = (String) simulationAgent.executeTransaction(t);
//		}else if (ranNum == 2) {
//			System.out.println("Returning without calling other components.");
//		}else if (ranNum == 3) {
//			System.out.println("Calling the DeploymentAdvisor component.");
//			t.setID((T) "STRATEGYANALYSIStoDEPLOYMENTADVISOR");
//			seqID = (String) deploymentAdvisor.executeTransaction(t);
//		}
		
		/* Perform the work of the method here */
		System.out.println("StrategyAnalysisAnomalyEngine executing its business logic.");
		try {Thread.sleep(1500);	// sleeping longer
		} catch (Exception e) {
			System.err.println("StrategyAnalysisAnomalyEngine Sleep exception:");
			e.printStackTrace();
		}
		
		// 1/2 chance we call strategyKB
//		ranNum = ran.nextInt(2);
//		if (ranNum != 0){
//			System.out.println("Calling the StrategyKB component.");
//			t.setID((T) "STRATEGYANALYSIStoSTRATEGYKB");
//			seqID = (String) strategyKB.executeTransaction(t);
//		}
		
		// 3/4 chance we call clock
		ranNum = ran.nextInt(4);
		if (ranNum != 0){
			System.out.println("Calling the Clock component.");
			t.setID((T) "STRATEGYANALYSIStoCLOCK");
			seqID = (String) clock.executeTransaction(t);
		}
		
		/* Try to log the exit from the method */
		try {
			// set the exit timestamp
			termDate = new Date();
			tmstpTerm = new java.sql.Timestamp(termDate.getTime());
			
			//init_pos = seq_vec.size() - 1;
			
			// find the most recent init values for this sequence id
			//while (!(seqID == seq_vec.elementAt(init_pos))){
			//	init_pos--;				
			//}
			
			// now update with the real original values
			//tranID = tran_vec.remove(init_pos);
			//seqID = seq_vec.remove(init_pos);
			//tmstpInit = tmstp_vec.remove(init_pos);
			
			// set the value for tran_init_tmstp, tran_term_tmstp, tran_id respectively
			wrapStmt.setLong(1, tmstpInit.getTime());
			wrapStmt.setLong(2, tmstpTerm.getTime());
			wrapStmt.setString(3, tranID);
			wrapStmt.setString(4, seqID);
			// execute the SQL insert statement
			wrapStmt.executeUpdate();
		} catch (Exception e) {
			System.err.println("StrategyAnalysisAnomalyEngine LogExit exception:");
			e.printStackTrace();
		}
		
		// exit from the method
		return (T) seqID;
	}

	
	/***********************************************************************************
	 * main()
	 * 
	 * This method runs when the object is first started up.
	 * This method should do all of the basic one-time setup for the object.
	 * This object will remain active because it will be referenced by the RMI registry.
	 **********************************************************************************/
	public static void main(String[] args) {
		if (System.getSecurityManager() == null){
			System.setSecurityManager(new SecurityManager());
		}
		
		cap = 0;
		// start sooper
		if (cont.sooper()) {cap = 4;}
		// end sooper
		
		/* Try to register this object in the RMI registry */
		try {
			String name = "rmi://EDSMine/StrategyAnalysisAnomaly";
			StrategyAnalysis engine = new StrategyAnalysisAnomalyEngine();
			StrategyAnalysis stub = (StrategyAnalysis) UnicastRemoteObject.exportObject(engine, 0);
			Registry registry = LocateRegistry.getRegistry();
			registry.rebind(name, stub);
			System.out.println("StrategyAnalysisAnomalyEngine bound");
		} catch (Exception e) {
			System.err.println("StrategyAnalysisAnomalyEngine RMI registration exception:");
			e.printStackTrace();
		}
		
		/* Try to connect to the database */
		try {
			// This will load the MySQL driver, each DB has its own driver
			Class.forName("com.mysql.jdbc.Driver");
			// Setup the connection with the DB
			connect = DriverManager.getConnection("jdbc:mysql://127.0.0.1/test?" + "user=edsmineman&password=edspass");
			// create the prepared statement for the transaction_init table insertion
			initStmt = connect.prepareStatement("insert into TEST.TRAN_INIT (id, tran_init_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?)");
			// create the prepared statement for the transaction_wrap table insertion
			wrapStmt = connect.prepareStatement("insert into TEST.TRAN_WRAP (id, tran_init_tmstp_l, tran_term_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?, ?)");
			
		} catch (Exception e) {
			System.err.println("StrategyAnalysisAnomalyEngine DB exception:");
			e.printStackTrace();
		}
		
		/* Try to get a hold of components needed through RMI */
		try {
			registry = LocateRegistry.getRegistry(args[0]);
			// start sooper
			if (cont.sooper()){
				clock = (Clock) registry.lookup(clockName);
				map = (Map) registry.lookup(mapName);
				resourceManager = (ResourceManager) registry.lookup(resourceManagerName);
				//weatherAnalyzer = (WeatherAnalyzer) registry.lookup(weatherAnalyzerName);
				//strategyKB = (StrategyKB) registry.lookup(strategyKBName);
			}// end sooper
		} catch (Exception e) {
			setupSuccessful = false;
			System.err.println("StrategyAnalysisAnomalyEngine RMI lookup exception:");
			e.printStackTrace();
		}
	}
}
