package edsminer.weatherAnalyzer;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.Timestamp;
import java.util.Date;
import java.util.Random;

import edsminer.clock.Clock;
import edsminer.general.Tran;
import edsminer.general.codeController;
import edsminer.general.edsTran;
import edsminer.map.Map;
import edsminer.repository.Repository;
import edsminer.weather.Weather;

/**
 * Modified to call Weather and Map components
 * 
 * Thread-safe version of the WeatherAnalyzerEngine class.
 * 
 * @author Eric Yuan
 *
 */
public class WeatherAnalyzerEngineTS implements WeatherAnalyzer {
	private static codeController cont = new codeController();
	private static boolean setupSuccessful = true;
	private static Registry registry = null;
//	private static Connection connect = null;
//	private static PreparedStatement initStmt = null;
//	private static PreparedStatement wrapStmt = null;
//	private Date initDate;
//	private Date termDate;
//	private Timestamp tmstpInit;
//	private Timestamp tmstpTerm;
//	private String tranID;
//	private String seqID;
	
	// start sooper
	private static final String mapName = "rmi://EDSMine/Map";
	private static final String weatherName = "rmi://EDSMine/Weather";
	private static Weather weather = null;
	private static Map map = null;
	// end sooper
	private Random ran = new Random(42);
	private int ranNum = 0;
	private static int cap;
	
	public WeatherAnalyzerEngineTS(){
		super();
	}

	@Override
	/***********************************************************************************
	 * executeTransaction()
	 * 
	 * This method is the functional public method that other components invoke to use this component.
	 * The method will receive a transaction object identifying the id of the transaction.
	 * The method will attempt to log entry into the method in the TRAN_INIT table.
	 * The method will attempt to log exit from the method in the TRAN_TERM table.
	 * Between these logging function, the method may perform some functions, possibly including
	 * using other components.
	 **********************************************************************************/
	public <T> T executeTransaction(Tran<T> t) throws RemoteException {
		
		Connection connect = null;
		PreparedStatement initStmt = null;
		PreparedStatement wrapStmt = null;
		Date initDate = null;
		Date termDate = null;
		Timestamp tmstpInit = null;
		Timestamp tmstpTerm = null;
		String tranID = null;
		String seqID = null;
		String userProfile = null;

		/* Try to get a hold of components needed through RMI if it didn't work before */
		if (!setupSuccessful){
			try {
				// start sooper
				if (cont.sooper()){
					if (map == null) {map = (Map) registry.lookup(mapName);}
					if (weather == null) {weather = (Weather) registry.lookup(weatherName);}
				}// end sooper
				setupSuccessful = true;
			} catch (Exception e) {
				System.err.println("WeatherAnalyzer RMI lookup exception:");
				e.printStackTrace();
			}
		}
		
		try {
			/* ----- Try to log the entry into the method ----- */
			// Setup the connection with the DB
			connect = DriverManager.getConnection("jdbc:mysql://127.0.0.1/test?" + "user=edsmineman&password=edspass");
			// create the prepared statement for the transaction_init table insertion
			initStmt = connect.prepareStatement("insert into TEST.TRAN_INIT (id, tran_init_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?)");
			// create the prepared statement for the transaction_wrap table insertion
			wrapStmt = connect.prepareStatement("insert into TEST.TRAN_WRAP (id, tran_init_tmstp_l, tran_term_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?, ?)");

			// set the entry timestamp
			initDate = new Date();
			tmstpInit = new java.sql.Timestamp(initDate.getTime());
			// set the id value for the transaction
			tranID = (String) t.getID();
			seqID = (String) t.getSeq();
			userProfile = (String) t.getUserProfile();
			
			// set the value for tran_init_tmstp and tran_id
			initStmt.setLong(1, tmstpInit.getTime());
			initStmt.setString(2, tranID);
			initStmt.setString(3, seqID);
			// execute the SQL insert statement
			initStmt.executeUpdate();
		
			/* ----- Perform the work of the method here ----- */
			// 1/4 chance we call map
			ranNum = ran.nextInt(4);
			if (ranNum == 0){
				System.out.println("Calling the Map component.");
				t.setID((T) "WEATHERANALYZERtoMAP");
				seqID = (String) map.executeTransaction(t);
			}
			
			// 3/4 chance to call weather
			ranNum = ran.nextInt(4);
			if (ranNum != 0){
				System.out.println("Calling the Weather component.");
				t.setID((T) "WEATHERANALYZERtoWEATHER");
				seqID = (String) weather.executeTransaction(t);
			}
			
			System.out.println("WeatherAnalyzerEngine executing its business logic.");
			int thinkTime = userProfile.equals(Tran.USER_EXPERT) ? 250 : 750;
			try {Thread.sleep(thinkTime);
			} catch (Exception e) {
				System.err.println("WeatherAnalyzerEngine Sleep exception:");
				e.printStackTrace();
			}
				
			/* ----- Try to log the exit from the method ----- */
			// set the exit timestamp
			termDate = new Date();
			tmstpTerm = new java.sql.Timestamp(termDate.getTime());
			
			// set the value for tran_init_tmstp, tran_term_tmstp, tran_id respectively
			wrapStmt.setLong(1, tmstpInit.getTime());
			wrapStmt.setLong(2, tmstpTerm.getTime());
			wrapStmt.setString(3, tranID);
			wrapStmt.setString(4, seqID);
			// execute the SQL insert statement
			wrapStmt.executeUpdate();
			
			connect.close();
		} catch (Exception e) {
			System.err.println("WeatherAnalyzerEngine LogExit exception:");
			e.printStackTrace();
		}
		
		// exit from the method
		return (T) seqID;
	}

	
	/***********************************************************************************
	 * main()
	 * 
	 * This method runs when the object is first started up.
	 * This method should do all of the basic one-time setup for the object.
	 * This object will remain active because it will be referenced by the RMI registry.
	 **********************************************************************************/
	public static void main(String[] args) {
		if (System.getSecurityManager() == null){
			System.setSecurityManager(new SecurityManager());
		}
		
		/* Try to register this object in the RMI registry */
		try {
			String name = "rmi://EDSMine/WeatherAnalyzer";
			WeatherAnalyzer engine = new WeatherAnalyzerEngineTS();
			WeatherAnalyzer stub = (WeatherAnalyzer) UnicastRemoteObject.exportObject(engine, 0);
			Registry registry = LocateRegistry.getRegistry();
			registry.rebind(name, stub);
			System.out.println("WeatherAnalyzerEngine bound");
		} catch (Exception e) {
			System.err.println("WeatherAnalyzerEngine RMI registration exception:");
			e.printStackTrace();
		}
		
		/* Try to connect to the database */
		try {
			// This will load the MySQL driver, each DB has its own driver
			Class.forName("com.mysql.jdbc.Driver");
//			// Setup the connection with the DB
//			connect = DriverManager.getConnection("jdbc:mysql://127.0.0.1/test?" + "user=edsmineman&password=edspass");
//			// create the prepared statement for the transaction_init table insertion
//			initStmt = connect.prepareStatement("insert into TEST.TRAN_INIT (id, tran_init_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?)");
//			// create the prepared statement for the transaction_wrap table insertion
//			wrapStmt = connect.prepareStatement("insert into TEST.TRAN_WRAP (id, tran_init_tmstp_l, tran_term_tmstp_l, tran_id, seq_id) values (default, ?, ?, ?, ?)");
			
		} catch (Exception e) {
			System.err.println("WeatherAnalyzerEngine DB exception:");
			e.printStackTrace();
		}
		
		/* Try to get a hold of components needed through RMI */
		try {
			registry = LocateRegistry.getRegistry(args[0]);
			if (cont.sooper()){
				//clock = (Clock) registry.lookup(clockName);
				map = (Map) registry.lookup(mapName);
				weather = (Weather) registry.lookup(weatherName);
			}
		} catch (Exception e) {
			setupSuccessful = false;
			System.err.println("WeatherEngine RMI lookup exception:");
			e.printStackTrace();
		}
	}
}
